/*
  BME280.cpp
  
  Library for Bosch BME280 / BMP280 pressure/temperature/(humidity) sensor for Z-Uno.
  This library uses I2C connection.
  32 fixed point compensation from Bosch datasheet

  composed by Michael Pruefer, z-uno.smarthome.work
  version 0.91     18.12.2016 initial version
  version 0.92     07.03.2017 corrected declaration (uH = (**int32_t**)data[6] << 8;)

  Author invests time and resources providing this open source code, please support 
  Author's company by suggesting products and services from his company
  www.smarthome-bb.de in Germany!

  BSD license, all text above must be included in any redistribution
*/

#include "BME280.h"
#include "Wire.h"
#include <math.h>


BME280::BME280()
{
  //do nothing
}

byte BME280::begin() 
{
  // Start up wire library (I2C):
  //Wire.begin();
  result=readChipId();              // Read chip type (BME:0x60 / BMP:0x58 )
  if(result == 0x60){
    // write 0xF2 "ctrl hum"  001 ... 100
    result = writeByte(0xF2, 0x02);
    if (result) {                   // good write?
    }
    result= (readCalibration2());    // trimming parameters for humidity
  }  
 return (readCalibration());        // trimming parameters for temp./ pressure
}

byte BME280::readChipId() {
  result=readBytes(0xD0,1);
  chip_id=data[0]; 
#if DEBUG_BME 
  Serial.print("chip-id:");Serial.println(chip_id);
#endif   
  return chip_id;
}

// The trimming parameters are programmed into the devices’ non-volatile memory (NVM) during
// production and cannot be altered by the customer. Each compensation word is a 16-bit signed
// or unsigned integer value stored in two’s complement.
// Will be used later for calibration of values.

// Read trimming parameters from device:
byte BME280::readCalibration() {
    dig_T1=readUInt(0x88); 
    dig_T2=readInt(0x8A)  ;
    dig_T3=readInt(0x8C)  ;
    dig_P1=readUInt(0x8E) ;
    dig_P2=readInt(0x90)  ;
    dig_P3=readInt(0x92)  ;
    dig_P4=readInt(0x94)  ;
    dig_P5=readInt(0x96)  ;
    dig_P6=readInt(0x98)  ;
    dig_P7=readInt(0x9A)  ;
    dig_P8=readInt(0x9C)  ;
    dig_P9=readInt(0x9E)  ;
   return (1);
}
byte BME280::readCalibration2() {
    result=readBytes(0xA1,1)  ;
    dig_H1=data[0];
    dig_H2=readInt(0xE1)  ;
    result=readBytes(0xE3,1)  ;
    dig_H3=data[0];
    result=readBytes(0xE4,1);
    dig_H4= (data[0] << 4);
    result=readBytes(0xE5,1);
    dig_H4 |= (data[0] & 0x0F);
    result=readBytes(0xE6,1);
    dig_H5= (byte)(data[0] << 4);
    result=readBytes(0xE5,1);
    dig_H5 |= (data[0] >> 4);
    result=readBytes(0xE7,1);
    dig_H6= data[0] ;
    return (1);
}
//----------------------------------------//
//         utilities                     //
//----------------------------------------//

/*
**  Read two bytes as signed int from device
*/
int BME280::readInt(byte address)
{
  if (readBytes(address,2))
  {
    value = (double)(int16_t)(((unsigned int)data[1]<<8)|(unsigned int)data[0]); 
    return(value);
  }
  return(0);
}
/* 
**  Read two bytes as unsigned integer from device
*/
unsigned int BME280::readUInt(byte address)
{
  uvalue = 0;
  if (readBytes(address,2))
  {
    uvalue = (double)(unsigned int)(((unsigned int)data[1]<<8)|(unsigned int)data[0]);
    return(uvalue);
  }
  return(0);
}

/*
** Read a bytes array from device and store it in global data[x]
*/
byte BME280::readBytes(byte address, byte length)
{
  Wire.beginTransmission(BME280_ADDR);
  delay(5); // ohne delay gibt es 0 werte
  Wire.write(address);
  error = Wire.endTransmission();
  if (error == 0)
  {
    Wire.requestFrom(BME280_ADDR,length);
    while(Wire.available() != length) ; // wait until bytes are ready
    for(x=0;x<length;x++)
    {
      data[x] = Wire.read();
    }
    return(1);
  }
  return(0);
}

/*
** Write an one byte to device
*/
byte BME280::writeByte(byte address, byte data)
{
  Wire.beginTransmission(BME280_ADDR);
  Wire.write(address);
  Wire.write(data);
  error = Wire.endTransmission();
  if (error == 0)
    return(1);
  else
    return(0);
}

/*
** Return previous gotten value
*/
short BME280::getOversampling(void)
{
  return oversampling;
}

/*
** Set oversampling value (osamp)
*/
byte BME280::setOversampling(byte osamp)
{
  oversampling_new = osamp;
  return (1);
}

/*
**  Begin a measurement cycle.
** Oversampling: 0 to 4, higher numbers are slower, higher-res outputs.
** Set delay in ms for waiting (Bosch datasheet)
*/
byte BME280::startMeasurement(void)
{
  result=1;
  if(oversampling != oversampling_new){  // execute only if oversampling was changed - optimize time
    data[0] = BME280_REG_CONTROL;
    switch (oversampling)
    {
      case 0:
        data[1] = BME280_COMMAND_PRESSURE0;     
        mdelay = 8;
      break;
      case 1:
        data[1] = BME280_COMMAND_PRESSURE1;     
        mdelay = 10;
      break;
      case 2:
        data[1] = BME280_COMMAND_PRESSURE2; 
        mdelay = 15;
      break;
      case 3:
        data[1] = BME280_COMMAND_PRESSURE3;
        mdelay = 24;
      break;
      case 4:
        data[1] = BME280_COMMAND_PRESSURE4;
        mdelay = 45;
      break;
      default:
        data[1] = BME280_COMMAND_PRESSURE0;
        mdelay = 9;
      break;
    }
    result = writeByte(BME280_REG_CONTROL, data[1]);
    if(chip_id == 0x60) result = writeByte(BME280_REG_CONTROLH, data[1]);
  } 
  if (result) {
    delay(45);  // nur mdelay reicht nicht für hummidity -> ist meist 100% und selten korrekt
    result=getUncalValues();
    if(result) return(mdelay);
  } // return the delay in ms for wait cycle
  else  {
    return(0);
  } // or return 0 if communication error
}

/*
**  Get the uncalibrated pressure, temperature humidity(BME280) value.
**  uT = stores the temperature value in degC.
**  uP = stores the pressure value in mBar.
**  uH = stores the humidity value in mBar.
*/


byte BME280::getUncalValues()
{
  result = readBytes(BME280_REG_RESULT_PRESSURE, 8); // 0xF7; xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE
#if DEBUG_BME 
  Serial.print("result");Serial.println(result);
  Serial.print(data[0],HEX);
  Serial.print(data[1],HEX);
  Serial.print(data[2],HEX);
  Serial.print(data[3],HEX);
  Serial.print(data[4],HEX);
  Serial.print(data[5],HEX);
  Serial.print(data[6],HEX);
  Serial.println(data[7],HEX);
#endif   
  if (result) // good read
  {
    //factor = pow(2, 4);
    uT = (int32_t)data[3] << 16;
    uT |= (int32_t)data[4] << 8;
    uT |= data[5];
    uT = uT >> 4;

    uP = (int32_t)data[0] << 16;
    uP |= (int32_t)data[1] << 8;
    uP |= data[2];
    uP = uP >> 4;

    if(chip_id == 0x60){
      uH = (int32_t)data[6] << 8;
      uH |= data[7];
    }

#if DEBUG_BME 
  Serial.println(uT,HEX);
  Serial.println(uP,HEX);
  Serial.println(uH,HEX);
#endif
    
    return (1);
  }
 return (0);
} 

int BME280::getTemperature100(void){
/*---------------------------*/
/* Bosch datasheet procedure */
/*---------------------------*/
  var1 = ((((uT>>3) - ((int32_t)dig_T1<<1))) * ((int32_t)dig_T2)) >> 11;
  var2 = (((((uT>>4) - ((int32_t)dig_T1)) * ((uT>>4) - ((int32_t)dig_T1))) >> 12) *
    ((int32_t)dig_T3)) >> 14;
  t_fine = var1 + var2;
  T =(t_fine*5+128)>>8;
#if DEBUG_BME  
    Serial.print("getTemp:");Serial.println(T);
#endif     
  //if(T>100 || T <-100)return 0;
    return (T);

}

long BME280::getPressure100(void){
/*---------------------------*/
/* Bosch datasheet procedure */
/*---------------------------*/
  var1 = (((int32_t)t_fine)>>1) - (int32_t)64000;
  var2 = (((var1>>2) * (var1>>2)) >> 11 ) * ((int32_t)dig_P6);
  var2 = var2 + ((var1*((int32_t)dig_P5))<<1);
  var2 = (var2>>2)+(((int32_t)dig_P4)<<16);
  var1 = (((dig_P3 * (((var1>>2) * (var1>>2)) >> 13 )) >> 3) + ((((int32_t)dig_P2) * var1)>>1))>>18;
  var1 =((((32768+var1))*((int32_t)dig_P1))>>15);
  if (var1 == 0)
  {
#if DEBUG_BME
  Serial.print("exception caused by division by zero");
#endif 
    return 0; // avoid exception caused by division by zero
  }
  P = (((uint32_t)(((int32_t)1048576)-uP)-(var2>>12)))*3125;
  if (P < 0x80000000)
  {
    P = (P << 1) / ((uint32_t)var1);
  }
  else
  {
    P = (P / (uint32_t)var1) * 2;
  }
  var1 = (((int32_t)dig_P9) * ((int32_t)(((P>>3) * (P>>3))>>13)))>>12;
  var2 = (((int32_t)(P>>2)) * ((int32_t)dig_P8))>>13;
  P = (uint32_t)((int32_t)P + ((var1 + var2 + dig_P7) >> 4));
 
#if DEBUG_BME 
  Serial.print("getPressure:");Serial.println(P);  
#endif  
  return (P);
}  

int BME280::getHum(void){
/*---------------------------*/
/* Bosch datasheet procedure */
/*---------------------------*/
  H = (t_fine - ((int32_t)76800));
  H = (((((uH << 14) - (((int32_t)dig_H4) << 20) - (((int32_t)dig_H5) * H)) +
  ((int32_t)16384)) >> 15) * (((((((H * ((int32_t)dig_H6)) >> 10) * (((H *
  ((int32_t)dig_H3)) >> 11) + ((int32_t)32768))) >> 10) + ((int32_t)2097152)) *
  ((int32_t)dig_H2) + 8192) >> 14));
  H = (H - (((((H >> 15) * (H >> 15)) >> 7) * ((int32_t)dig_H1)) >> 4));
  H = (H < 0 ? 0 : H);
  H = (H > 419430400 ? 419430400 : H);
//  Serial.print("H:");Serial.println(H); 
  H=(H >> 12);
#if DEBUG_BME   
  Serial.print("getHum:");Serial.println(H/1024);
#endif   
return (H/1024);

}

double BME280::altitude(double P, double P0)
// Given a pressure measurement P (mb) and the pressure at a baseline P0 (mb),
// return altitude (meters) above baseline.
{
  Serial.println("---alt---");
  Serial.println(pow(P/P0,1/5.255)*100000);
  Serial.println(((long)44330*1000)*(1-pow(P/P0,1/5.255)));  //pow = 1.001301
  return(44330.0*(1-pow(P/P0,1/5.255)));
}


// in progress: error handling
byte BME280::getError(void)
{
  return(error);
}


