/*
  BME280.h
  
  Library for Bosch BME280 / BMP280 pressure/temperature/(humidity) sensor for Z-Uno.
  This library uses I2C connection.
  32 fixed point compensation from Bosch datasheet

  composed by Michael Pruefer, z-uno.smarthome.work
  version 0.91     18.12.2016 initial version

  Author invests time and resources providing this open source code, please support 
  Author's company by suggesting products and services from his company
  www.smarthome-bb.de in Germany!

  BSD license, all text above must be included in any redistribution
*/

#ifndef BME280_h
#define BME280_h

#include "Arduino.h"

#define DEBUG_BME 0

class BME280
{
  public:
    BME280(); // base type

    byte begin();
        
    short getOversampling(void);
    byte  setOversampling(byte osamp);
    
    byte startMeasurement(void);
      // command BME280 to start a pressure measurement
      // oversampling: 0 - 3 for oversampling value
      // returns (number of ms to wait) for success, 0 for fail
    
    double altitude(double P, double P0);
      // convert absolute pressure to altitude (given baseline pressure; sea-level, runway, etc.)
      // P: absolute pressure (mbar)
      // P0: fixed baseline pressure (mbar)
      // returns signed altitude in meters

    byte getError(void);
      // If command fails, error code using this command can be used.
      // in progress
      
    byte readChipId(); 
      // Read chip type (BME:0x60 / BMP:0x58 )

    byte getUncalValues();
      //get uncalibrated Pressure (uP) and Temperature (uT) value.
      
    int getTemperature100(void);
      // Read the calibrated Temperature value in Centi-Grad Celsius 
      // (Celsius * 100 -> 21.25 degree Celsius will be 2125)
      
    long getPressure100(void);
      // Read the calibrated Pressurerature value in Pascal 
      // (Pascal * 100 -> 1021.25 hPa  will be 102125)
      
    int getHum(void);
      // Read the calibrated Humidity value in % 
    
      float T; 
      int32_t H, P, var1, var2;
      byte chip_id;


  private:
  
    byte readCalibration();    // Temp, Pressure
    byte readCalibration2();   // Humdity if available
      // Retrieve calibration data from device:
      // The trimming parameters are programmed into the devices’ non-volatile memory (NVM) during
      // production and cannot be altered by the customer. Each compensation word is a 16-bit signed
      // or unsigned integer value stored in two’s complement.


    int readInt(byte address);
      // read an signed int (16 bits) from a BME280 register
      // address: BME280 register address
      // returns with result in uvalue success, 0 for fail + error value will be set (in progress)

    unsigned int readUInt(byte address);
      // read an unsigned int (16 bits) from a BME280 register
      // address: BME280 register address
      // returns with result in uvalue success, 0 for fail + error value will be set (in progress)

    byte readBytes(byte address, byte length);
      // read a given number of bytes from a BME280 register
      // values: array of byte with register address in first location [0]
      // length: number of bytes to read back
      // returns 1 for success, 0 for fail + error value will be set (in progress), result in data[] array
      
    byte writeByte(byte address, byte data);
      // write a byte to a BME280 register
      // address: BME280 register address
      // data: byte to be written
      // returns 1 for success, 0 for fail + error value will be set (in progress)
    
        
    int dig_T2, dig_T3, dig_T4, dig_P2 , dig_P3, dig_P4, dig_P5, dig_P6, dig_P7, dig_P8, dig_P9, dig_H2, dig_H4, dig_H5, dig_H6; 
    unsigned int dig_T1, dig_P1, dig_H1, dig_H3; 
    byte error, x;
    short oversampling, oversampling_new;

    int32_t uP, uT, uH, t_fine, factor;
    int32_t data[8];
    unsigned int uvalue;
    int value;
    unsigned char result, mdelay;
};

#define BME280_ADDR 0x76 // 7-bit address

#define BME280_REG_CONTROLH 0xF2
#define BME280_REG_CONTROL 0xF4
#define BME280_REG_RESULT_PRESSURE 0xF7     // 0xF7(msb) , 0xF8(lsb) , 0xF9(xlsb) : pressure data.
#define BME280_REG_RESULT_TEMPRERATURE 0xFA   // 0xFA(msb) , 0xFB(lsb) , 0xFC(xlsb) : temperature data.
#define BME280_REG_RESULT_HUMIDITY 0xFD   // 0xFD(msb) , 0xFE(lsb) : humidity data.

#define BME280_COMMAND_TEMPERATURE 0x2E
#define BME280_COMMAND_PRESSURE0 0x25        
#define BME280_COMMAND_PRESSURE1 0x29       
#define BME280_COMMAND_PRESSURE2 0x2D    
#define BME280_COMMAND_PRESSURE3 0x31    
#define BME280_COMMAND_PRESSURE4 0x5D    

#endif

